from .misfit import (
    API_URL,
    Misfit,
    MisfitProfile,
    MisfitDevice,
    MisfitGoal,
    MisfitSummary,
    MisfitSession,
    MisfitSleepDetail,
    MisfitSleep
)

__all__ = ['API_URL', 'Misfit', 'MisfitProfile', 'MisfitDevice', 'MisfitGoal',
           'MisfitSummary', 'MisfitSession', 'MisfitSleepDetail',
           'MisfitSleep']
__title__ = 'misfit'
__author__ = 'ORCAS'
__author_email__ = 'bpitcher@orcasinc.com'
__copyright__ = 'Copyright 2014 ORCAS'
__license__ = 'Apache 2.0'
__version__ = '0.1.1'



#!/usr/bin/env python
"""Misfit client

Usage:
  misfit authorize --client_id=<client_id> --client_secret=<client_secret> [--config=<config_file>]
  misfit profile [--user_id=<user_id>] [--object_id=<object_id>] [--config=<config_file>]
  misfit device [--user_id=<user_id>] [--object_id=<object_id>] [--config=<config_file>]
  misfit goal (--start_date=<start_date> --end_date=<end_date>|--object_id=<object_id>) [--user_id=<user_id>] [--config=<config_file>]
  misfit summary --start_date=<start_date> --end_date=<end_date> [--detail] [--user_id=<user_id>] [--config=<config_file>]
  misfit session (--start_date=<start_date> --end_date=<end_date>|--object_id=<object_id>) [--user_id=<user_id>] [--config=<config_file>]
  misfit sleep (--start_date=<start_date> --end_date=<end_date>|--object_id=<object_id>) [--user_id=<user_id>] [--config=<config_file>]
  misfit --version

Options:
  -h --help                        Show this screen.
  --version                        Show version.
  --client_id=<client_id>          App key of your Misfit app.
  --client_secret=<client_secret>  App secret of your Misfit app.
  --config=<config_file>           Use the config file specified [default: ./misfit.cfg]
  --user_id=<user_id>              Misfit User ID.
  --object_id=<object_id>          ID of a single Misfit object.
  --start_date=<start_date         Date at the start of a range: Eg. 2014-11-20.
  --end_date=<end_date>            Date at the end of a range: Eg. 2014-11-30.
  --detail                         If specified, print summary detail for each day.

"""
from __future__ import absolute_import

from docopt import docopt
from pprint import PrettyPrinter

from . import __version__
from .auth import MisfitAuth
from .misfit import Misfit

try:
    import configparser
except ImportError:  # Python 2.x fallback
    import ConfigParser as configparser


class MisfitCli:
    def __init__(self, arguments):
        """
        Runs the command specified as an argument with the options specified
        """
        self.config_file = arguments['--config']
        self.config = configparser.ConfigParser()
        self.client_id = None
        self.client_secret = None
        self.access_token = None

        if arguments['authorize']:
            self.client_id = arguments['--client_id']
            self.client_secret = arguments['--client_secret']
            self.authorize()
        elif not arguments['--version']:
            try:
                # Fail if config file doesn't exist or is missing information
                self.read_config()
            except (IOError, configparser.NoOptionError,
                    configparser.NoSectionError):
                print('Missing config information, please run '
                      '"misfit authorize"')
            else:
                # Everything is good! Get the requested resource(s)
                self.get_resource(arguments)

    def read_config(self):
        """ Read credentials from the config file """
        with open(self.config_file) as cfg:
            self.config.readfp(cfg)
        self.client_id = self.config.get('misfit', 'client_id')
        self.client_secret = self.config.get('misfit', 'client_secret')
        self.access_token = self.config.get('misfit', 'access_token')

    def write_config(self, access_token):
        """ Write credentials to the config file """
        self.config.add_section('misfit')
        self.config.set('misfit', 'client_id', self.client_id)
        self.config.set('misfit', 'client_secret', self.client_secret)
        self.config.set('misfit', 'access_token', access_token)
        with open(self.config_file, 'w') as cfg:
            self.config.write(cfg)
        print('Credentials written to %s' % self.config_file)

    def get_resource(self, arguments):
        """ Gets the resource requested in the arguments """
        user_id = arguments['--user_id']
        object_id = arguments['--object_id']
        start_date = arguments['--start_date']
        end_date = arguments['--end_date']
        detail = arguments['--detail']

        misfit = Misfit(self.client_id, self.client_secret, self.access_token,
                        user_id)

        if arguments['profile']:
            result = misfit.profile(object_id)
        elif arguments['device']:
            result = misfit.device(object_id)
        elif arguments['goal']:
            result = misfit.goal(start_date, end_date, object_id)
        elif arguments['summary']:
            result = misfit.summary(start_date, end_date, detail)
        elif arguments['session']:
            result = misfit.session(start_date, end_date, object_id)
        elif arguments['sleep']:
            result = misfit.sleep(start_date, end_date, object_id)
        pp = PrettyPrinter(indent=4)
        if isinstance(result, list):
            pp.pprint([res.data for res in result])
        else:
            pp.pprint(result.data)

    def authorize(self):
        """
        Authorize a user using the browser and a CherryPy server, and write
        the resulting credentials to a config file.
        """

        # Thanks to the magic of docopts, I can be guaranteed to have a
        # a client_id and client_secret
        auth = MisfitAuth(self.client_id, self.client_secret)
        auth.browser_authorize()

        # Write the authentication information to a config file for later use
        if auth.token:
            self.write_config(auth.token['access_token'])
        else:
            print('ERROR: We were unable to authorize to use the Misfit API.')


def main():
    """ Parse the arguments and use them to create a MisfitCli object """
    version = 'Python Misfit %s' % __version__
    arguments = docopt(__doc__, version=version)
    MisfitCli(arguments)


if __name__ == '__main__':
    """ Makes this file runnable with "python -m misfit.cli" """
    main()



import arrow
import json
import requests

from base64 import standard_b64decode
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15

from .misfit import MisfitObject


def string_to_sign(data):
    """ Build a signed SNS string from a dict """
    strings = []
    for key in ('Message', 'MessageId', 'Subject', 'SubscribeURL', 'Timestamp',
                'Token', 'TopicArn', 'Type'):
        if key in data:
            strings += [key, data[key]]
    strings.append("")
    return '\n'.join(strings).encode('utf8')


class MisfitNotification(MisfitObject):
    def __init__(self, data):
        """ Load the JSON to a dict and verify the signature if applicable """
        data_dict = json.loads(data.decode('utf8'))
        super(MisfitNotification, self).__init__(data_dict)
        if hasattr(self, 'Signature'):
            self.verify_signature()
        if self.Type == 'Notification':
            # Load the messages into a list of dicts, with arrow objects for
            # dates
            messages = json.loads(self.Message)
            for i, message in enumerate(messages):
                messages[i]['updatedAt'] = arrow.get(message['updatedAt'])
            self.Message = messages
        elif self.Type == 'SubscriptionConfirmation':
            # If the notification is a subscription confirmation, fetch the
            # subscribe URL
            requests.get(self.SubscribeURL)

    def verify_signature(self):
        """
        Verify the signature of the SNS message.

        raises cryptography.exceptions.InvalidSignature
        """
        # Get the signing certificate and public key from the specified URL
        cert_url = self.data['SigningCertURL']
        cert_str = requests.get(cert_url).content.encode('utf8')
        cert = default_backend().load_pem_x509_certificate(cert_str)
        pubkey = cert.public_key()
        # Verify the signature
        signature = standard_b64decode(self.data['Signature'].encode('utf8'))
        verifier = pubkey.verifier(signature, PKCS1v15(), hashes.SHA1())
        verifier.update(string_to_sign(self.data))
        # verify returns None on success, raises InvalidSignature on failure
        assert verifier.verify() is None



from __future__ import absolute_import

import cherrypy
import os
import threading
import webbrowser

from oauthlib.oauth2 import Client
from oauthlib.oauth2.rfc6749.errors import MismatchingStateError
from requests_oauthlib import OAuth2Session

from .misfit import API_URL


class MisfitAuth:
    def __init__(self, client_id, client_secret,
                 redirect_uri='http://127.0.0.1:8080/', state=None,
                 scope=['public', 'birthday', 'email'], success_html=None,
                 failure_html=None):
        """ Initialize the OAuth2Session """
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.scope = scope
        self.success_html = success_html if success_html else """
            <h1>You are now authorized to access the Misfit API!</h1>
            <br/><h3>You can close this window</h3>"""
        self.failure_html = failure_html if failure_html else """
            <h1>ERROR: We were unable to authorize to use the Misfit API.</h1>
            <br/><h3>You can close this window</h3>"""
        # Ignore when the Misfit API doesn't return the actual scope granted,
        # even though this goes against rfc6749:
        #     https://github.com/idan/oauthlib/blob/master/oauthlib/oauth2/rfc6749/parameters.py#L392
        os.environ['OAUTHLIB_RELAX_TOKEN_SCOPE'] = 'true'
        self.token = None
        self.state = state
        self.oauth = OAuth2Session(
            self.client_id, scope=self.scope, redirect_uri=self.redirect_uri,
            state=self.state)

    def authorize_url(self):
        """
        Build the authorization url and save the state. Return the
        authorization url
        """
        url, self.state = self.oauth.authorization_url(
            '%sauth/dialog/authorize' % API_URL)
        return url

    def fetch_token(self, code, state):
        """
        Fetch the token, using the verification code. Also, make sure the state
        received in the response matches the one in the request. Returns the
        access_token.
        """
        if self.state != state:
            self._shutdown_cherrypy()
            raise MismatchingStateError()
        self.token = self.oauth.fetch_token(
            '%sauth/tokens/exchange/' % API_URL, code=code,
            client_secret=self.client_secret)
        return self.token['access_token']

    def browser_authorize(self):
        """
        Open a browser to the authorization url and spool up a CherryPy
        server to accept the response
        """
        webbrowser.open(self.authorize_url())
        cherrypy.quickstart(self)

    @cherrypy.expose
    def index(self, state, code=None, error=None):
        """
        Receive a Misfit response containing a verification code. Use the code
        to fetch the access_token.
        """
        if code:
            self.fetch_token(code, state)
        self._shutdown_cherrypy()
        return self.success_html if code else self.failure_html

    def _shutdown_cherrypy(self):
        if cherrypy.engine.state == cherrypy.engine.states.STARTED:
            threading.Timer(1, lambda: cherrypy.engine.exit()).start()



import json


class MisfitException(Exception):
    pass


class MisfitHttpException(MisfitException):
    """
    Base class for all HTTP exceptions raised by the Misfit API. All exceptions
    will have three attributes.

    - code: The status code of the HTTP response, defaults to 500
    - message: The messages returned from the Misfit API
    - response: If we have one, this attribute will be the response object, of
      type :code:`requests.models.Response`. It's useful to have access to this
      because each response contains useful headers. For example, if the
      library throws an exception of type :code:`MisfitRateLimitError` you can
      retrieve a time stamp for when the rate limit resets with the following
      code: :code:`exc.response.headers['x-ratelimit-reset']`
    """
    def __init__(self, code, message, response=None):
        self.code = code
        self.message = message
        self.response = response
        super(MisfitHttpException, self).__init__(self, message)

    @staticmethod
    def build_exception(exc):
        code = exc.response.status_code if hasattr(exc, 'response') else 500
        message = exc.message if hasattr(exc, 'message') else 'Unknown error'
        try:
            json_content = json.loads(exc.content)
        except ValueError:
            pass
        else:
            # Loading the content to json succeeded, try to get the
            # code/message from there
            if 'message' in json_content:
                message = json_content['message']
            elif 'error_message' in json_content:
                message = json_content['error_message']
            if 'code' in json_content:
                code = json_content['code']
            elif 'error_code' in json_content:
                code = json_content['error_code']

        exceptions = {
            400: MisfitBadRequest,
            401: MisfitUnauthorized,
            403: MisfitForbidden,
            404: MisfitNotFoundError,
            429: MisfitRateLimitError,
            500: MisfitUnknownError,
            502: MisfitBadGateway
        }
        raise exceptions[code](code, message, getattr(exc, 'response', None))


class MisfitNotFoundError(MisfitHttpException):
    pass


class MisfitBadRequest(MisfitHttpException):
    pass


class MisfitBadGateway(MisfitHttpException):
    pass


class MisfitUnauthorized(MisfitHttpException):
    pass


class MisfitForbidden(MisfitHttpException):
    pass


class MisfitRateLimitError(MisfitHttpException):
    pass


class MisfitUnknownError(MisfitHttpException):
    pass



import arrow
import json
import slumber
import sys

from oauthlib.oauth2 import Client
from requests_oauthlib import OAuth2
from slumber.exceptions import HttpClientError, HttpServerError
from slumber.serialize import Serializer, JsonSerializer

from .exceptions import MisfitException, MisfitHttpException

API_URL = 'https://api.misfitwearables.com/'


class MisfitSerializer(JsonSerializer):
    """ Override the built-in JSON serializer to handle bytes """
    def loads(self, data):
        return json.loads(data.decode('utf8'))


class Misfit:
    def __init__(self, client_id, client_secret, access_token, user_id=None):
        auth = OAuth2(client_id, Client(client_id),
                      {'access_token': access_token})
        user = user_id if user_id else 'me'
        s = Serializer(default="json", serializers=[MisfitSerializer()])
        self.api = slumber.API('%smove/resource/v1/user/%s/' % (API_URL, user),
                               auth=auth, serializer=s)

    def profile(self, object_id=None):
        return MisfitProfile(self._get_object(self.api.profile, object_id))

    def device(self, object_id=None):
        return MisfitDevice(self._get_object(self.api.device, object_id))

    def goal(self, start_date=None, end_date=None, object_id=None):
        self._check_date_range_or_id(start_date, end_date, object_id)
        goals = self._get_object(
            self.api.activity.goals, object_id,
            start_date=start_date, end_date=end_date)
        if 'goals' in goals:
            return [MisfitGoal(goal) for goal in goals['goals']]
        return MisfitGoal(goals)

    def summary(self, start_date, end_date, detail=False):
        summary = self._get_object(
            self.api.activity.summary, start_date=start_date,
            end_date=end_date, detail='true' if detail else 'false')
        if 'summary' in summary:
            return [MisfitSummary(summ) for summ in summary['summary']]
        return MisfitSummary(summary)

    def session(self, start_date=None, end_date=None, object_id=None):
        self._check_date_range_or_id(start_date, end_date, object_id)
        sessions = self._get_object(
            self.api.activity.sessions, object_id,
            start_date=start_date, end_date=end_date)
        if 'sessions' in sessions:
            return [MisfitSession(session) for session in sessions['sessions']]
        return MisfitSession(sessions)

    def sleep(self, start_date=None, end_date=None, object_id=None):
        self._check_date_range_or_id(start_date, end_date, object_id)
        sleeps = self._get_object(
            self.api.activity.sleeps, object_id,
            start_date=start_date, end_date=end_date)
        if 'sleeps' in sleeps:
            return [MisfitSleep(sleep) for sleep in sleeps['sleeps']]
        return MisfitSleep(sleeps)

    def _check_date_range_or_id(self, start_date, end_date, object_id):
        if (start_date is None or end_date is None) and object_id is None:
            raise MisfitException(
                'Either a date range or object id must be supplied')

    def _get_object(self, api_section, object_id=None, **kwargs):
        try:
            args = (object_id,) if object_id else tuple()
            return api_section(*args).get(**kwargs)
        except (HttpClientError, HttpServerError):
            MisfitHttpException.build_exception(sys.exc_info()[1])


class UnicodeMixin(object):
    if sys.version_info > (3, 0):
        __str__ = lambda x: x.__unicode__()
    else:
        __str__ = lambda x: unicode(x).encode('utf-8')


class MisfitObject(UnicodeMixin):
    def __init__(self, data):
        self.data = data
        for name, value in self.data.items():
            self.set_value(name, value)

    def set_value(self, name, value):
        if name in ['date', 'datetime', 'startTime', 'Timestamp']:
            setattr(self, name, arrow.get(value))
        else:
            setattr(self, name, value)

    def __unicode__(self):
        return '%s: %s' % (type(self), json.dumps(self.data))


class MisfitProfile(MisfitObject): pass


class MisfitDevice(MisfitObject): pass


class MisfitGoal(MisfitObject):
    def percent_complete(self):
        """
        Using the number of points received, and comparing to the target
        points, return the percent complete (from 0-100). Returns
        :code:`None` if the target is 0.
        """
        if self.targetPoints == 0:
            return None
        return float(self.points) / self.targetPoints * 100


class MisfitSummary(MisfitObject): pass


class MisfitSession(MisfitObject): pass


class MisfitSleepDetail(MisfitObject): pass


class MisfitSleep(MisfitObject):
    def __init__(self, data):
        super(MisfitSleep, self).__init__(data)
        sleep_details = []
        for sleep_detail in self.sleepDetails:
            sleep_details.append(MisfitSleepDetail(sleep_detail))
        self.sleepDetails = sleep_details



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import multiprocessing
import re

from setuptools import setup


required = [l for l in open('requirements/base.txt').read().split("\n")]
required_test = [l for l in open('requirements/test.txt').read().split("\n")]

mfinit = open('misfit/__init__.py').read()
refind = lambda varname: re.search("%s = '([^']+)'" % varname, mfinit).group(1)

setup(
    name='misfit',
    version=refind('__version__'),
    description='Misfit API client implementation',
    long_description=open('README.rst').read(),
    author=refind('__author__'),
    author_email=refind('__author_email__'),
    url='https://github.com/orcasgit/python-misfit',
    packages=['misfit'],
    package_data={'': ['LICENSE']},
    include_package_data=True,
    install_requires=["setuptools"] + required,
    license=refind('__license__'),
    entry_points={
        'console_scripts': ['misfit=misfit.cli:main'],
    },
    test_suite='nose.collector',
    tests_require=required_test,
    classifiers=(
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy'
    ),
)
